==> kom%2B%2B-0.3.tar.gz: kom++-0.3/libkom++/unread-confs.cc <==
// Handle information about conferences with unread texts.
//
// $Id: unread-confs.cc,v 1.21 1994/08/08 22:00:32 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <iostream.h>
#include <stdlib.h>

#include "unread-confs.h"
#include "connection.h"
#include "get_unread_confs_q.h"
#include "pepsi_q.h"

Unread_info::Unread_info()
    :conf(0), min_unread(-1), max_unread(-2)
{
}

Unread_info &
Unread_info::operator =(const Unread_info &u)
{
  conf = u.conf;
  min_unread = u.min_unread;
  max_unread = u.max_unread;
  return *this;
}

// ================================================================ 
//                    Constructors and Destructors                  


Unread_confs::Unread_confs(Session *sess, Pers_no the_reader)
    : session(sess), conferences(sess->conferences()), 
      persons(sess->persons()), maps(sess->maps()), texts(sess->texts()),
      unread_confs(), 
      reader(the_reader), read_confs(), current_working_conf(0),
      ucq(NULL), unread_confs_has_been_fetched(FALSE)
{
}
 
// FIXME: Make Unread_confs into a reference counting class.
// This class owns the Read_orders pointed to by read_confs and unread_confs,
// so we cannot have two Unread_confs objects pointing to the same Read_orders.
//
// Unread_confs::Unread_confs(const Unread_confs &cl)
//     : session(cl.session), conferences(cl.conferences), persons(cl.persons),
//       maps(cl.maps), texts(cl.texts), unread_confs(cl.unread_confs), 
//       reader(cl.reader), read_confs(cl.read_confs),
//       current_working_conf(cl.current_working_conf), ucq(cl.ucq),
//       unread_confs_has_been_fetched(cl.unread_confs_has_been_fetched)
// {
// }


Unread_confs::~Unread_confs()
{
    Pix ix;

    for (ix = unread_confs.first(); ix != 0; unread_confs.next(ix))
	delete unread_confs(ix);
    for (ix = read_confs.first(); ix != 0; read_confs.next(ix))
	delete read_confs(ix);

    delete ucq;
}


// ================================================================ 
//

//const Unread_confs & 
//Unread_confs::operator = (const Unread_confs &)
//{
//    cerr << "Operator= for class Unread_confs is not yet implemented\n";
//    return *this;
//}


// ================================================================
//

int
Unread_confs::prefetch_map(int required_texts,
			int text_in_confs,
			int confs,
			int max_question,
			int min_question)
{
    if (!unread_confs_has_been_fetched)
    {
	ucq = new get_unread_confs_question(conferences.connection(), reader);
	unread_confs_has_been_fetched = TRUE;
    }

    if (ucq != NULL && ucq->receive() == st_pending)
	return 1;

    if (ucq != NULL)
    {
	if (ucq->receive() == st_error)
	   return -1;

	assert(unread_confs.empty() && read_confs.empty());

	Array<Conf_no> new_unread_confs = ucq->result();
	delete ucq;
	ucq = NULL;

	for (int conf_ix = 0; conf_ix < new_unread_confs.size(); conf_ix++)
	{
	    add_conference(new_unread_confs[conf_ix], TRUE);
	}
    }

    int questions = 0;

    // Step 1: make sure we know about at least REQUIRED_TEXTS texts.

    for (Pix ix = unread_confs.first();
	 ix != 0 && required_texts > 0 && questions < max_question;
	 unread_confs.next(ix))
    {
	int nq = unread_confs(ix)->prefetch_map(required_texts);
	int nt = unread_confs(ix)->min_num_unread();

	required_texts -= nt;
	if (nq > 0)
	    questions += nq;
	
	if (nq == 0 && nt == 0)
	{     
	    // There is nothing to read in this conference. Remove it
	    // from unread_confs.
	    read_confs.append(unread_confs(ix));
	    unread_confs.del(ix, -1);
	}
    }
    
    // Step 2: Know about at least TEXT_IN_CONFS unread texts in at
    // least CONF conferences.

    for (ix = unread_confs.first();
	 ix != 0 && confs > 0 && questions < max_question;
	 unread_confs.next(ix))
    {
	int nq = unread_confs(ix)->prefetch_map(text_in_confs);
	int nt = unread_confs(ix)->min_num_unread();

	confs--;
	
	if (nq > 0)
	    questions += nq;
	
	if (nq == 0 && nt == 0)
	{     
	    // There is nothing to read in this conference. Remove it
	    // from unread_confs.
	    read_confs.append(unread_confs(ix));
	    unread_confs.del(ix, -1);
	}
    }

    // Step 3: Issue at least MIN_QUESTION questions.

    for (ix = unread_confs.first();
	 ix != 0 && questions < min_question;
	 unread_confs.next(ix))
    {
	int nq = unread_confs(ix)->prefetch_map(999999); // FIXME+++
	int nt = unread_confs(ix)->min_num_unread();

	if (nq > 0)
	    questions += nq;
	
	if (nq == 0 && nt == 0)
	{     
	    // There is nothing to read in this conference. Remove it
	    // from unread_confs.
	    read_confs.append(unread_confs(ix));
	    unread_confs.del(ix, -1);
	}
    }
    
    return questions;
}

int
Unread_confs::prefetch(int required_texts,
		    int text_in_confs,
		    int confs,
		    int max_question,
		    int min_question)
{
    int questions = prefetch_map(required_texts, text_in_confs, confs,
				 max_question, min_question);

    for (Pix ix = unread_confs.first();
	 ix != 0 && required_texts > 0 && questions < max_question; 
	 unread_confs.next(ix))
    {
	Read_order *r = unread_confs(ix);
	int nq = r->prefetch(required_texts, max_question-questions, 
			     min_question-questions);
	int nt = r->min_num_unread();

	required_texts -= nt;
	if (nq > 0)
	    questions += nq;
    }

    for (ix = unread_confs.first();
	 ix != 0 && confs > 0 && questions < max_question;
	 unread_confs.next(ix))
    {
	Read_order *r = unread_confs(ix);
	int nq = r->prefetch(text_in_confs, max_question-questions, 
			     min_question-questions);
	confs--;
	
	if (nq > 0)
	    questions += nq;
    }

    for (ix = unread_confs.first();
	 ix != 0 && questions < min_question;
	 unread_confs.next(ix))
    {
	Read_order *r = unread_confs(ix);
	int nq = r->prefetch(999999, /// FIXME+++
			     max_question-questions, 
			     min_question-questions);
	if (nq > 0)
	    questions += nq;
    }

    return questions;
}

Conf_no
Unread_confs::goto_next_conf()
{
    while (prefetch_map(1,0,0,10,0) > 0)
	if (Connection::drain(100) == st_error)
	    return 0;

    if (unread_confs.empty())
	return current_working_conf;

    // Unread_confs::prefetch should remove any empty Read_order.
    assert (unread_confs.front()->empty() == FALSE);

    // Are we leaving this conference before everything is unread? If
    // so, move it to the tail of unread conferences.
    if (current_working_conf == unread_confs.front()->confno())
	unread_confs.append(unread_confs.remove_front()); 

    while (prefetch_map(1,0,0,10,0) > 0)
	if (Connection::drain(100) == st_error)
	    return 0;

    assert (unread_confs.front()->empty() == FALSE);

    if (current_working_conf != unread_confs.front()->confno())
    {
	current_working_conf = unread_confs.front()->confno();
	pepsi_question q(conferences.connection(), current_working_conf);
	q.receive();		// +++ Ignore this question.
    }

    return current_working_conf;
}

Status
Unread_confs::goto_conf(Conf_no c)
{
    Read_order *ro = NULL;

    Pix ix;

    for (ix = unread_confs.first(); ix != 0; unread_confs.next(ix))
	if (unread_confs(ix)->confno() == c)
	{
	    ro = unread_confs(ix);
	    unread_confs.del(ix);
	    break;
	}

    if (ro == NULL)
	for (ix = read_confs.first(); ix != 0; read_confs.next(ix))
	    if (read_confs(ix)->confno() == c)
	    {
		ro = read_confs(ix);
		read_confs.del(ix);
		break;
	    }

    if (ro == NULL)
	ro = new Read_order(session, c , reader);

    pepsi_question q(conferences.connection(), c);
    // +++ We could relatively safely ignore this question, except
    // when we had to create a new Read_order.
    if (q.receive() == st_ok)
    {
	unread_confs.prepend(ro);
	current_working_conf = c;
    }
    else
    {
	// The user was not a member in this conference.
	delete ro;
    }
    return q.receive();
}

Prompt
Unread_confs::next_prompt()
{
    Prompt   prompt;

    while (prefetch_map(1,0,0,10,0) > 0)
	if (Connection::drain(100) == st_error)
	    return PROMPT_ERROR;

    if (unread_confs.empty())
	return NO_MORE_TEXT;

    if (current_working_conf != unread_confs.front()->confno())
	return NEXT_CONF;

    prompt = unread_confs.front()->next_prompt();
    assert(prompt != NO_MORE_TEXT && prompt != NEXT_CONF);
    return prompt;
}


Text_no
Unread_confs::next_text(Bool remove)
{
    while (prefetch_map(1,0,0,10,0) > 0)
	if (Connection::drain(100) == st_error)
	    return 0;

    if (unread_confs.empty())
	return 0;

    Text_no  next = unread_confs.front()->next_text(remove);
    if (next && remove)
	remove_text(next);

    return next;
}

void
Unread_confs::remove_text(Text_no tno)
{
    // Loop through the recipients of the text.
    for (int i = 0; i < texts[tno].num_recipients(); ++i) {
	Conf_no  cno = texts[tno].recipient(i)->recipient();

        // Loop through all unread_confs and if we find the recipient,
        // then remove the text from it.
	Pix ix;
        for (ix = unread_confs.first(); ix != 0; unread_confs.next(ix)) {
	    if (unread_confs(ix)->confno() == cno)
		unread_confs(ix)->remove_text(tno);
	}
	// We must check read_confs as well, because the user may have
	// moved the conference there with Unread_confs::goto_next_conf().
        for (ix = read_confs.first(); ix != 0; read_confs.next(ix)) {
	    if (read_confs(ix)->confno() == cno)
		read_confs(ix)->remove_text(tno);
	}
    }
}


Conf_no
Unread_confs::current_conf()
{
    return current_working_conf;
}

Array<Unread_info>
Unread_confs::unread_texts()
{
    Array<Unread_info>   arr(unread_confs.length());
    int                  index = 0;

    for (Pix ix = unread_confs.first(); ix != 0; unread_confs.next(ix)) 
    {
	arr[index].conf = unread_confs(ix)->confno();
	arr[index].min_unread = unread_confs(ix)->min_num_unread();
	arr[index].max_unread = unread_confs(ix)->max_num_unread();
	index++;
    }

    return arr;
}

long
Unread_confs::min_unread_texts_in_current()
{
    if (unread_confs.empty())
	return 0;
    return unread_confs.front()->min_num_unread();

}

long
Unread_confs::max_unread_texts_in_current()
{
    if (unread_confs.empty())
	return 0;
    return unread_confs.front()->max_num_unread();
}

void
Unread_confs::add_conference(Conf_no cno, Bool last)
{
    Read_order  * read_order;

    read_order = new Read_order(session, cno, reader);
    if (last)
	unread_confs.append(read_order);
    else
	unread_confs.prepend(read_order);
    // Note: If there were no texts in this conference, it will be
    // moved to read_confs as soon as we start to prefetch from it.
}

Status
Unread_confs::join_conference(Conf_no cno, 
			      unsigned char priority,
			      unsigned short placement)
{
    // FIXME: There should be a way to insert the new conference in the
    // correct place. We do not know the order of the conferences. Big
    // lose.+++

    Pix ix;
    Bool present = FALSE;
    for (ix = unread_confs.first(); ix != 0 && !present; unread_confs.next(ix))
	if (unread_confs(ix)->confno() == cno)
	    present = TRUE;
    for (ix = read_confs.first(); ix != 0 && !present; read_confs.next(ix))
	if (read_confs(ix)->confno() == cno)
	    present = TRUE;
    
    Status ret = session->add_member(cno, reader, priority, placement);

    if (ret != st_ok)
    {
	// FIXME: Errorcode
	return ret;
    }

    if (!present)
	add_conference(cno, TRUE);
    else
    {
	// FIXME: Update placement & priority in the local cache.
    }

    return ret;
}
// eof

==> kom%2B%2B-0.4.tar.gz: kom++-0.4/libkom++/unread-confs.cc <==
// Handle information about conferences with unread texts.
//
// $Id: unread-confs.cc,v 1.30 1995/12/30 16:26:51 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <iostream.h>
#include <stdlib.h>

#include "unread-confs.h"
#include "connection.h"
#include "get_unread_confs_q.h"
#include "pepsi_q.h"
#include "async_text_created.h"

Unread_info::Unread_info()
    :conf(0), min_unread(-1), max_unread(-2)
{
}

Unread_info &
Unread_info::operator =(const Unread_info &u)
{
  conf = u.conf;
  min_unread = u.min_unread;
  max_unread = u.max_unread;
  return *this;
}

// ================================================================ 
//                    Constructors and Destructors                  


Unread_confs::Unread_confs(Session *sess, Pers_no the_reader)
    : session(sess),
      conferences(sess->conferences()), 
      persons(sess->persons()),
      maps(sess->maps()),
      texts(sess->texts()),
      unread_confs(), 
      read_confs(),
      reader(the_reader),
      current_working_conf(0),
      ucq(NULL),
      cr_tag(sess->async_dispatcher().register_text_created_handler
	     (async_callback_method(this, &Unread_confs::new_text))),
      unread_confs_has_been_fetched(false)
{
}
 
// FIXME: Make Unread_confs into a reference counting class.
// This class owns the Read_orders pointed to by read_confs and unread_confs,
// so we cannot have two Unread_confs objects pointing to the same Read_orders.
//
// Unread_confs::Unread_confs(const Unread_confs &cl)
//     : session(cl.session), conferences(cl.conferences), persons(cl.persons),
//       maps(cl.maps), texts(cl.texts), unread_confs(cl.unread_confs), 
//       reader(cl.reader), read_confs(cl.read_confs),
//       current_working_conf(cl.current_working_conf), ucq(cl.ucq),
//       unread_confs_has_been_fetched(cl.unread_confs_has_been_fetched)
// {
// }


Unread_confs::~Unread_confs()
{
    Pix ix;

    for (ix = unread_confs.first(); ix != 0; unread_confs.next(ix))
	delete unread_confs(ix);
    for (ix = read_confs.first(); ix != 0; read_confs.next(ix))
	delete read_confs(ix);

    session->async_dispatcher().unregister_text_created_handler(cr_tag);

    delete ucq;
}


// ================================================================ 
//

//const Unread_confs & 
//Unread_confs::operator = (const Unread_confs &)
//{
//    cerr << "Operator= for class Unread_confs is not yet implemented\n";
//    return *this;
//}


// ================================================================
//

Status
Unread_confs::prefetch(Work_quota *work)
{
    int q = work->questions_left();

    if (!unread_confs_has_been_fetched)
    {
	ucq = new get_unread_confs_question(conferences.connection(), reader);
	unread_confs_has_been_fetched = true;
    }

    if (ucq != NULL && ucq->receive() == st_pending)
    {
	work->bump_question();
	return st_pending;
    }

    if (ucq != NULL)
    {
	if (ucq->receive() == st_error)
	   return st_error;

	assert(unread_confs.empty() && read_confs.empty());

	Array<Conf_no> new_unread_confs = ucq->result();
	delete ucq;
	ucq = NULL;

	for (int conf_ix = 0; conf_ix < new_unread_confs.size(); conf_ix++)
	{
	    add_conference(new_unread_confs[conf_ix], true);
	}
    }

    Pix ix;
    for (ix = unread_confs.first();
	 ix != 0 && !work->done();
	 unread_confs.next(ix))
    {
	work->change_conf();
	Status st = unread_confs(ix)->prefetch(work);
	int nt = unread_confs(ix)->min_num_unread();

	if (st == st_ok && nt == 0)
	{     
	    // There is nothing to read in this conference. Remove it
	    // from unread_confs.
	    read_confs.append(unread_confs(ix));
	    unread_confs.del(ix, -1);
	}
    }

    return (q == work->questions_left()) ? st_ok : st_pending;
}

Conf_no
Unread_confs::goto_next_conf()
{
    if (need_one_mapped() == st_error)
	return 0;

    if (unread_confs.empty())
	return current_working_conf;

    // Unread_confs::prefetch should remove any empty Read_order.
    assert (unread_confs.front()->empty() == false);

    // Are we leaving this conference before everything is unread? If
    // so, move it to the tail of unread conferences.
    if (current_working_conf == unread_confs.front()->confno())
	unread_confs.append(unread_confs.remove_front()); 

    if (need_one_mapped() == st_error)
	return 0;

    assert (unread_confs.front()->empty() == false);

    if (current_working_conf != unread_confs.front()->confno())
    {
	current_working_conf = unread_confs.front()->confno();
	pepsi_question q(conferences.connection(), current_working_conf);
	q.receive();		// +++ Ignore this question.
    }

    return current_working_conf;
}

Status
Unread_confs::goto_conf(Conf_no c)
{
    if (current_working_conf == c)
	// Return immediately if we are already standing in the conference.
	return st_ok;

    Read_order *ro = NULL;

    Pix ix;

    if (need_unread_confs() != st_ok)
	return st_error;

    for (ix = unread_confs.first(); ix != 0; unread_confs.next(ix))
	if (unread_confs(ix)->confno() == c)
	{
	    ro = unread_confs(ix);
	    unread_confs.del(ix);
	    break;
	}

    if (ro == NULL)
	for (ix = read_confs.first(); ix != 0; read_confs.next(ix))
	    if (read_confs(ix)->confno() == c)
	    {
		ro = read_confs(ix);
		read_confs.del(ix);
		break;
	    }

    if (ro == NULL)
	ro = new Read_order(session, c , reader);

    pepsi_question q(conferences.connection(), c);
    // +++ We could relatively safely ignore this question, except
    // when we had to create a new Read_order.
    if (q.receive() == st_ok)
    {
	unread_confs.prepend(ro);
	current_working_conf = c;
    }
    else
    {
	// The user was not a member in this conference.
	delete ro;
    }
    return q.receive();
}

Prompt
Unread_confs::next_prompt()
{
    Prompt   prompt;

    if (need_one_mapped() == st_error)
	return PROMPT_ERROR;

    if (unread_confs.empty())
	return NO_MORE_TEXT;

    if (current_working_conf != unread_confs.front()->confno())
	return NEXT_CONF;

    prompt = unread_confs.front()->next_prompt();
    assert(prompt != NO_MORE_TEXT && prompt != NEXT_CONF);
    return prompt;
}


Text_no
Unread_confs::next_text(bool remove)
{
    if (need_one_mapped() == st_error)
	return 0;

    if (unread_confs.empty())
	return 0;

    Text_no  next = unread_confs.front()->next_text(remove);
    if (next != 0 && remove)
	remove_text(next);

    return next;
}

void
Unread_confs::remove_text(Text_no tno)
{
    if (need_unread_confs() != st_ok)
	return;

    // Loop through the recipients of the text.
    for (int i = 0; i < texts[tno].num_recipients(); ++i) {
	Conf_no  cno = texts[tno].recipient(i)->recipient();
	Local_text_no lno = texts[tno].recipient(i)->local_no();

        // Loop through all unread_confs and if we find the recipient,
        // then remove the text from it.
	Pix ix;
        for (ix = unread_confs.first(); ix != 0; unread_confs.next(ix)) {
	    if (unread_confs(ix)->confno() == cno)
		unread_confs(ix)->remove_text(tno, lno);
	}
    }
}


Conf_no
Unread_confs::current_conf()
{
    return current_working_conf;
}

Array<Unread_info>
Unread_confs::confs_with_unread()
{
    if (need_unread_confs() != st_ok)
	return 0;

    Array<Unread_info>   arr(unread_confs.length());
    int                  index = 0;

    for (Pix ix = unread_confs.first(); ix != 0; unread_confs.next(ix)) 
    {
	arr[index].conf = unread_confs(ix)->confno();
	arr[index].min_unread = unread_confs(ix)->min_num_unread();
	arr[index].max_unread = unread_confs(ix)->max_num_unread();
	index++;
    }

    return arr;
}

long
Unread_confs::min_unread_texts_in_current()
{
    if (need_unread_confs() != st_ok)
	return 0;

    if (unread_confs.empty())
	return 0;
    return unread_confs.front()->min_num_unread();

}

long
Unread_confs::max_unread_texts_in_current()
{
    if (need_unread_confs() != st_ok)
	return 0;

    if (unread_confs.empty())
	return 0;
    return unread_confs.front()->max_num_unread();
}

Read_order_iterator
Unread_confs::current_conf_begin(int wanted_texts)
{
    need_one_mapped();
    return unread_confs.front()->begin(wanted_texts);
}

Read_order_iterator
Unread_confs::current_conf_end()
{
    need_one_mapped();
    return unread_confs.front()->end();
}

void
Unread_confs::add_conference(Conf_no cno, bool last)
{
    if (need_unread_confs() != st_ok)
	return;

    Read_order  * read_order;

    read_order = new Read_order(session, cno, reader);
    if (last)
	unread_confs.append(read_order);
    else
	unread_confs.prepend(read_order);
    // Note: If there were no texts in this conference, it will be
    // moved to read_confs as soon as we start to prefetch from it.
}

Status
Unread_confs::join_conference(Conf_no cno, 
			      unsigned char priority,
			      unsigned short placement)
{
    // FIXME: There should be a way to insert the new conference in the
    // correct place. We do not know the order of the conferences. Big
    // lose.+++

    if (need_unread_confs() != st_ok)
	return st_error;

    Pix ix;
    bool present = false;
    for (ix = unread_confs.first(); ix != 0 && !present; unread_confs.next(ix))
	if (unread_confs(ix)->confno() == cno)
	    present = true;
    for (ix = read_confs.first(); ix != 0 && !present; read_confs.next(ix))
	if (read_confs(ix)->confno() == cno)
	    present = true;
    
    Status ret = session->add_member(cno, reader, priority, placement);

    if (ret != st_ok)
    {
	// FIXME: Errorcode
	return ret;
    }

    if (!present)
	add_conference(cno, true);
    else
    {
	// FIXME: Update placement & priority in the local cache.
    }

    return ret;
}

void
Unread_confs::new_text(const Async_text_created &msg)
{
    if (need_unread_confs() != st_ok)
	return;

    Text_stat t = msg.text_stat();
    for (int ir = 0; ir < t.num_recipients(); ir++)
    {
	Recipient *rcpt = t.recipient(ir);
	Conf_no confno = rcpt->recipient();
	bool present = false;

	for (Pix ix = unread_confs.first(); ix != 0 && !present;
	     unread_confs.next(ix))
	{
	    if (unread_confs(ix)->confno() == confno)
		present = true;
	}
	if (!present)
	{
	    for (Pix iy = read_confs.first(); iy != 0; read_confs.next(iy))
		if (read_confs(iy)->confno() == confno)
		{
		    unread_confs.append(read_confs(iy));
		    read_confs.del(iy);
		    present = true;
		    break;
		}
	}

	if (!present)
	    add_conference(rcpt->recipient(), true);
    }
}

Status
Unread_confs::need_one_mapped()
{
    while (prefetch(&Work_quota(1,0,10)) == st_pending)
	if (Connection::drain(100) == st_error)
	    return st_error;
    return st_ok;
}

Status
Unread_confs::need_unread_confs()
{
    while (!unread_confs_has_been_fetched
	   && prefetch(&Work_quota(1,0,10)) == st_pending)
    {
	if (Connection::drain(100) == st_error)
	    return st_error;
    }

    return unread_confs_has_been_fetched ? st_ok : st_error;
}

// eof
